from argparse import ArgumentParser
from functools import partial
from pandas import DataFrame
import sys

from tabulate import tabulate

from opa_tools import get_request_df, get_environ_df

def _get_parser():
    parser = ArgumentParser()
    subparsers = parser.add_subparsers(title="Service", dest="service")

    parser_req = subparsers.add_parser("req")
    _add_table_arguments(parser_req)

    parser_env = subparsers.add_parser("env")
    _add_table_arguments(parser_env)

    return parser

def _add_table_arguments(parser):
    parser.add_argument("-d", "--dates", action="store", nargs="*", default=[])
    parser.add_argument(
        "-f", "--format",
        action="store",
        type=str,
        choices=["default", "fwf"] + [
            method.lstrip("to_")
            for method
            in dir(DataFrame)
            if method.startswith("to_")
        ],
        default="default",
    )
    parser.add_argument(
        "-t", "--tablefmt",
        action="store",
        type=str,
        metavar="TABULAR_TABLEFMT",
        default="plain",
    )

def _table_service(args, fun):
    df = fun(*args.dates)
    fmt = args.format
    if fmt == "default":
        print(df)
        return
    if fmt == "fwf":
        print(
            tabulate(
                df.values.tolist(),
                list(df.columns),
                tablefmt=args.tablefmt
            )
        )
        return
    if hasattr(df, f"to_{fmt}"):
        print(getattr(df, f"to_{fmt}")())
        return
    raise ValueError(
        f"Unknown format: '{fmt}'."
    )

_AVAILABLE_SERVICES = {
    "req": partial(_table_service, fun=get_request_df),
    "env": partial(_table_service, fun=get_environ_df),
}

def bch(argv=None):
    if argv is None:
        argv = sys.argv[1:]
    
    parser = _get_parser()
    args = parser.parse_args(argv)
    service = args.service
    if not service:
        parser.print_help()
        sys.exit(1)

    _AVAILABLE_SERVICES[service](args)

